@@ -6,7 +6,7 @@ from typing import Dict, List, Optional, Tuple
 
 from ..flowerpower import FlowerPowerProject
 from ..pipeline.manager import HookType, PipelineManager
-from ..cfg.pipeline.run import RunConfig
+from ..cfg.pipeline.run import RunConfig, RetryConfig
 from ..utils.logging import setup_logging
 from .utils import parse_dict_or_list_param
 
@@ -42,6 +42,16 @@ def run(
     executor: str | None = typer.Option(
         None, help="Executor to use for running the pipeline"
     ),
+    executor_cfg: str | None = typer.Option(
+        None,
+        help='Executor configuration as JSON/dict string (e.g. "{\"type\":\"threadpool\",\"max_workers\":4}")',
+    ),
+    executor_max_workers: int | None = typer.Option(
+        None, help="Convenience flag: set executor max_workers"
+    ),
+    executor_num_cpus: int | None = typer.Option(
+        None, help="Convenience flag: set executor num_cpus"
+    ),
     base_dir: str | None = typer.Option(None, help="Base directory for the pipeline"),
     inputs: str | None = typer.Option(
         None, help="Input parameters as JSON, dict string, or key=value pairs"
@@ -63,7 +73,7 @@ def run(
         None, help="Adapter configuration as JSON or dict string"
     ),
     max_retries: int = typer.Option(
-        0, help="Maximum number of retry attempts on failure"
+        0, help="Maximum number of retry attempts on failure (0 to disable retries)"
     ),
     retry_delay: float = typer.Option(
         1.0, help="Base delay between retries in seconds"
@@ -125,6 +135,7 @@ def run(
     parsed_final_vars = parse_dict_or_list_param(final_vars, "list") or []
     parsed_storage_options = parse_dict_or_list_param(storage_options, "dict") or {}
     parsed_with_adapter = parse_dict_or_list_param(with_adapter, "dict") or {}
+    parsed_executor_cfg = parse_dict_or_list_param(executor_cfg, "dict") or {}
     
     # Ensure proper types for RunConfig
     if parsed_inputs is not None and not isinstance(parsed_inputs, dict):
@@ -168,14 +179,31 @@ def run(
             config=parsed_config,
             cache=parsed_cache,
             with_adapter=with_adapter_config,  # type: ignore
-            max_retries=max_retries,
-            retry_delay=retry_delay,
-            jitter_factor=jitter_factor,
+            retry=RetryConfig(
+                max_retries=max_retries,
+                retry_delay=retry_delay,
+                jitter_factor=jitter_factor,
+            ),
         )
-        
-        # Handle executor configuration
-        if executor is not None:
-            run_config.executor.type = executor
+
+        # Handle executor configuration (type + config + convenience flags)
+        try:
+            if executor is not None:
+                # Validate type
+                from ..utils.security import validate_executor_type
+                validate_executor_type(executor)
+                parsed_executor_cfg["type"] = executor
+            if executor_max_workers is not None:
+                parsed_executor_cfg["max_workers"] = executor_max_workers
+            if executor_num_cpus is not None:
+                parsed_executor_cfg["num_cpus"] = executor_num_cpus
+
+            if parsed_executor_cfg:
+                from ..cfg.pipeline.run import ExecutorConfig
+                run_config.executor = ExecutorConfig.from_dict(parsed_executor_cfg)
+        except Exception as e:
+            logger.error(f"Invalid executor configuration: {e}")
+            raise typer.Exit(1)
 
         _ = project.run(name=name, run_config=run_config)
         logger.info(f"Pipeline '{name}' finished running.")
