@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # mypy: disable-error-code="attr-defined"
 # pylint: disable=no-member, E1136, W0212, W0201
 """
@@ -7,8 +6,12 @@ Manages the import and export of pipelines.
 
 import posixpath
 
-from fsspec_utils import (AbstractFileSystem, BaseStorageOptions,
-                          DirFileSystem, filesystem)
+from fsspec_utils import (
+    AbstractFileSystem,
+    BaseStorageOptions,
+    DirFileSystem,
+    filesystem,
+)
 from loguru import logger
 from rich.console import Console
 
@@ -74,9 +77,13 @@ class PipelineIOManager:
     def _print_export_success(self, names: list[str] | None, dest_base_dir: str) -> None:
         """Print success message for export operations."""
         if names:
-            self._print_export_success(names, dest_base_dir)
+            console.print(
+                f"✅ Exported pipelines [bold blue]{', '.join([self.project_cfg.name + '.' + n for n in names])}[/bold blue] to [green]{dest_base_dir}[/green]"
+            )
         else:
-            self._print_export_success(None, dest_base_dir)
+            console.print(
+                f"✅ Exported all pipelines from [bold blue]{self.project_cfg.name}[/bold blue] to [green]{dest_base_dir}[/green]"
+            )
 
     def _sync_filesystem(
         self,
