@@ -1,14 +1,11 @@
-import datetime as dt
 import os
 import posixpath
 import sys
 import warnings
 from pathlib import Path
 from types import TracebackType
-from typing import Any, Callable, TypeVar, Union
-from uuid import UUID
+from typing import Any, TypeVar, Union
 
-from loguru import logger
 from munch import Munch
 
 try:
@@ -18,17 +15,16 @@ except ImportError:
 
 from fsspec_utils import AbstractFileSystem, BaseStorageOptions, filesystem
 
-from ..settings import CONFIG_DIR, PIPELINES_DIR, CACHE_DIR
 from ..cfg import PipelineConfig, ProjectConfig
 from ..cfg.pipeline.run import RunConfig
-from ..utils.logging import setup_logging
-from ..utils.config import merge_run_config_with_kwargs
+from ..settings import CACHE_DIR, CONFIG_DIR, PIPELINES_DIR
 from ..utils.filesystem import FilesystemHelper
+from ..utils.logging import setup_logging
 from .config_manager import PipelineConfigManager
 from .executor import PipelineExecutor
 from .io import PipelineIOManager
 from .lifecycle_manager import PipelineLifecycleManager
-from .registry import PipelineRegistry, HookType
+from .registry import HookType, PipelineRegistry
 from .visualizer import PipelineVisualizer
 
 setup_logging()
@@ -76,7 +72,6 @@ class PipelineManager:
         fs: AbstractFileSystem | None = None,
         cfg_dir: str | None = CONFIG_DIR,
         pipelines_dir: str | None = PIPELINES_DIR,
-        
         log_level: str | None = None,
     ) -> None:
         """Initialize the PipelineManager.
@@ -95,7 +90,7 @@ class PipelineManager:
                 Example: "config" or "settings".
             pipelines_dir: Override default pipelines directory name ('pipelines').
                 Example: "flows" or "dags".
-            
+
             log_level: Set logging level for the manager.
                 Valid values: "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"
 
@@ -115,7 +110,7 @@ class PipelineManager:
             ...         "key": "ACCESS_KEY",
             ...         "secret": "SECRET_KEY"
             ...     },
-            
+
             ...     log_level="DEBUG"
             ... )
         """
@@ -133,7 +128,7 @@ class PipelineManager:
         storage_options: dict | Munch | BaseStorageOptions | None,
         fs: AbstractFileSystem | None,
         cfg_dir: str | None,
-        pipelines_dir: str | None
+        pipelines_dir: str | None,
     ) -> None:
         """Setup filesystem and configuration directories.
 
@@ -163,13 +158,24 @@ class PipelineManager:
             cached = False
             cache_storage = None
 
-        # Get filesystem instance
-        self._fs = fs or self._fs_helper.get_filesystem(cached=cached, cache_storage=cache_storage)
-        self._storage_options = (
-            storage_options or self._fs.storage_options
-            if self._fs.protocol != "dir"
-            else self._fs.fs.storage_options
-        )
+        # Get filesystem instance (use local `filesystem` to allow tests to patch)
+        if fs is not None:
+            self._fs = fs
+        else:
+            self._fs = filesystem(
+                self._base_dir,
+                storage_options=(storage_options or {}),
+                cached=cached,
+                cache_storage=cache_storage,
+            )
+        try:
+            self._storage_options = (
+                storage_options or self._fs.storage_options
+                if getattr(self._fs, "protocol", None) != "dir"
+                else self._fs.fs.storage_options
+            )
+        except Exception:
+            self._storage_options = storage_options or {}
 
     def _initialize_managers(self) -> None:
         """Initialize all manager components."""
@@ -178,7 +184,7 @@ class PipelineManager:
             base_dir=self._base_dir,
             fs=self._fs,
             storage_options=self._storage_options,
-            cfg_dir=self._cfg_dir
+            cfg_dir=self._cfg_dir,
         )
 
         # Load project configuration
@@ -194,25 +200,21 @@ class PipelineManager:
 
         # Initialize specialized managers
         self._executor = PipelineExecutor(
-            config_manager=self._config_manager,
-            registry=self.registry
+            config_manager=self._config_manager, registry=self.registry
         )
         self._lifecycle_manager = PipelineLifecycleManager(registry=self.registry)
 
         # Initialize other components
         self._project_context = None
         self.visualizer = PipelineVisualizer(
-            project_cfg=self._config_manager.project_config,
-            fs=self._fs
+            project_cfg=self._config_manager.project_config, fs=self._fs
         )
         self.io = PipelineIOManager(registry=self.registry)
 
     def _ensure_directories_exist(self) -> None:
         """Ensure essential directories exist."""
         self._fs_helper.ensure_directories_exist(
-            self._fs,
-            self._cfg_dir,
-            self._pipelines_dir
+            self._fs, self._cfg_dir, self._pipelines_dir
         )
 
     def _add_modules_path(self) -> None:
@@ -282,7 +284,6 @@ class PipelineManager:
         # Add cleanup code if needed
         pass
 
-
     def load_pipeline(self, name: str, reload: bool = False) -> PipelineConfig:
         """Load or reload configuration for a specific pipeline.
 
@@ -346,12 +347,8 @@ class PipelineManager:
 
     # --- Core Execution Method ---
 
-
     def run(
-        self,
-        name: str,
-        run_config: RunConfig | None = None,
-        **kwargs
+        self, name: str, run_config: RunConfig | None = None, **kwargs
     ) -> dict[str, Any]:
         """Execute a pipeline synchronously and return its results.
 
@@ -426,7 +423,7 @@ class PipelineManager:
         # Set project context for executor
         if hasattr(self, "_project_context") and self._project_context is not None:
             self._executor._project_context = self._project_context
-        
+
         # Delegate to executor
         return self._executor.run(name=name, run_config=run_config, **kwargs)
 
@@ -593,10 +590,12 @@ class PipelineManager:
         try:
             if fmt == "json":
                 import json
+
                 print(json.dumps(names))
                 return None
             if fmt == "yaml":
                 import yaml  # type: ignore
+
                 print(yaml.safe_dump(names, sort_keys=False))
                 return None
         except Exception as e:
@@ -1047,8 +1046,13 @@ class PipelineManager:
             ...     reload=True
             ... )
         """
+
         return self.visualizer.save_dag(
-            name=name, format=format, reload=reload, output_path=output_path
+            name=name,
+            format=format,
+            reload=reload,
+            output_path=output_path,
+            base_dir=self._base_dir,
         )
 
     def show_dag(
